import os, json, csv, openpyxl, docx, docx.shared


class simplify_str:
    def __init__(self, value: str, casei=False):
        """Extract the value in the string and convert it
        into non-reference object(int,float)."""
        self.v = value
        self.i = casei

    def simplify(self):
        try:
            self.v = float(self.v)
        except ValueError:
            pass
        try:
            self.v = int(self.v)
        except ValueError:
            pass
        try:
            self.v = self.__toBool()
        except (ValueError, AttributeError):
            pass
        try:
            self.v = self.__toNone()
        except (ValueError, AttributeError):
            pass
        return self.v

    def __toNone(self):
        if self.i:
            if self.v.lower() == 'none':
                return None
            raise ValueError('Unknown None value represation.')
        else:
            if self.v == 'None':
                return None
            raise ValueError('Unknown None value represation.')

    def __toBool(self):
        if self.i:
            if self.v.lower() == 'true':
                return True
            elif self.v.lower() == 'false':
                return False
            raise ValueError('Unknown boolean value.')
        else:
            if self.v == 'True':
                return True
            elif self.v == 'False':
                return False
            raise ValueError('Unknown boolean value.')


class BaseFile:
    def __init__(self):
        self._opath = None
        self._path = None
        self._nl = ['\n']
        self._ne = [' ']
        self._dof = False
        self._ftext = []

    @property
    def text(self):
        """The string contained in the file."""
        with open(str(self._opath))as f:
            return f.read()

    def _get_ftext(self):
        temp, temp1, temp2 = [], '', []
        for i in self.text + self._nl[0]:
            if i in self._ne:
                temp2.append(temp1)
                temp1 = ''
            elif i in self._nl:
                temp2.append(temp1)
                temp.append(temp2)
                temp1 = ''
                temp2 = []
            else:
                temp1 += i
        return temp.copy()

    def write(self, obj: str):
        """Write to the original file."""
        with open(str(self._opath), 'w')as f:
            f.write(obj)
        self._ftext = self._get_ftext()
        return len(obj)

    def append(self, obj: str):
        """Append something to the original file."""
        with open(str(self._opath), 'a')as f:
            f.write(obj)
        self._ftext = self._get_ftext()
        return len(obj)

    def convert_to_xlsx(self):
        """Convert the file into Microsoft spread sheet."""
        self.__wb = openpyxl.Workbook()
        self.__sheet = self.__wb.get_sheet_by_name("Sheet")
        temp = 1
        for i in self._ftext:
            self.__write_row(i, temp)
            temp += 1
        self.__wb.save(self._path.full(no_ext=True) + '.xlsx')
        del self.__wb
        del self.__sheet

    def convert_to_csv(self, next_cell_token=',', next_row_token='\n'):
        """convert the text file into .csv file using csv module."""
        print(self._path.full(no_ext=True))
        writer = csv.writer(open(self._path.full(no_ext=True)+'.csv', 'w', newline=''), delimiter=next_cell_token,
                            lineterminator=next_row_token)
        for i in self._ftext:
            writer.writerow(i)

    def __write_row(self, list_to_write, start_row, start_column=1):
        try:
            self.__sheet.cell(row=start_row, column=start_column).value = list_to_write[start_column - 1]
            return self.__write_row(list_to_write, start_row, start_column=start_column + 1)
        except IndexError:
            return

    def convert_to_json(self):
        """Use json module to convert the file into .json file."""
        json.dump(self.text,open(self._path.full(no_ext=True)+'.json','w'))

    def close(self, ignore_error=False):
        """Close the file, return 0 if succeeded."""
        if ignore_error:
            try:
                if self._dof:
                    os.unlink(str(self._opath))
                return 0
            except:
                return 1
        else:
            if self._dof:
                os.unlink(str(self._opath))
            return 0

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def __str__(self):
        return self.text

    def convert_to_docx(self, font=13):
        """Convert to a .docx file."""
        doc = docx.Document()
        para = doc.add_paragraph(self.text)
        para.font.size = docx.shared.Pt(font)
        doc.save(self._path.full(no_ext=True) + '.docx')



import subprocess

import dill
import os

from pyil.enum.variable import pythonPath2
from .._coll import ProcessError, random_word


class Launcher:
    """There is a lot of things this class can't do like print(),
    so you shouldn't use this separately, it is made for background
    process."""

    def __init__(self, source: str, name='Process', args=None, kwargs=None, required=None):
        """This isn't a class you should use, but since this is the
        core of how the Thread class work, I wrote this doc string.
        Do not change any of the variables in this class, the result
        will be bad. Basically, how this work is that it creates
        another python file and run it to get the result. Since the
        GIL isn't shared between different python interpreter, this
        will achieve real threading.(It doesn't have to have
        if __name__=='__main__':
            pass
        ) source parameter is the function you want to execute.(Saved
        using dill)

        args and kwargs follows the API of threading module. required
        should contain a list of the required module in string."""
        assert (args is None or type(args) == tuple) and (kwargs is None or type(kwargs) == dict) and \
               (required is None or type(required) == list), "args or kwargs is not tuple or dictionary"
        self.__res = 'result' + name + '.dat'
        self.src = source
        self.__run_name = 'run' + name + '.pyw'
        self.req = required
        if required is None:
            self.req = ()
        self.args = "args_" + name + ".dat"
        self.kwargs = "kwargs_" + name + ".dat"
        self.__name = name
        if args==None:
            dill.dump((), open(self.args, 'wb'))
        else:
            dill.dump(args, open(self.args, 'wb'))
        if kwargs==None:
            dill.dump({}, open(self.kwargs, 'wb'))
        else:
            dill.dump(kwargs, open(self.kwargs, 'wb'))

    def end(self) -> int:
        """End the process and return 1."""
        subprocess.Popen.terminate(self.process)
        self.clean()
        return 1

    def prepare(self):
        """Prepare for the execution(Create python file)."""
        with open(self.__run_name, 'w')as f:
            f.write("#!python3\n")  # Shebang line
            for i in self.req:  # Throws all the imports
                f.write("import " + i + "\n")
            f.write("import dill,os\n")
            f.write("func=dill.load( open('" + self.src + "','rb') )\n")  # Get the function
            f.write("args=dill.load(open('" + self.args + "','rb'))\n")
            f.write("kwargs=dill.load(open('" + self.kwargs + "','rb'))\n")
            f.write("dill.dump(func(")  # Dump the result using dill
            f.write("*args,**kwargs")  # Dump the parameters.
            f.write("),open('" + self.__res + "','wb'))\n")

    def start(self):
        """Start the process, must call prepare() before starting."""
        self.process = subprocess.Popen([pythonPath2, self.__run_name])

    @property
    def finished(self):
        """Finished or not."""
        if self.process.poll() is not None:
            self.exit_code = self.process.poll()
            return True
        return False

    def wait(self):
        """Wait then return the exit code."""
        return self.process.wait()

    @property
    def result(self):
        """What's the result? If result isn't produced
        yet or an exception occurred will return a NoValue
        object(I did't choose to use None because sometimes
        the return value of a function maybe None)."""
        if self.finished:  # Check if finished.
            if self.exit_code != 0:  # An exception occurred
                self.clean()
                raise ProcessError('%s failed with exit code: %s' % (self.__name, self.exit_code))
            while True:  # There is a delay between the if the process finished
                if os.path.exists(self.__res):  # and the result file was produced.
                    while True:  # There is also another delay between if the
                        try:  # file was produced and if it is finished.
                            return dill.load(open(self.__res, 'rb'))
                        except EOFError:  # pickle will raise EOFError if the file is not completed
                            pass
        raise ProcessError('%s not finished yet.' % self.__name)

    def clean(self) -> int:
        """Clean up the file generated."""
        try:
            os.unlink(self.args)
            os.unlink(self.kwargs)
            os.unlink(self.__run_name)
            os.unlink(self.__res)
        except FileNotFoundError as a:
            return 1
        return 0

    def __enter__(self):
        self.prepare()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end()



import os as _os
import dill as _dill
from .__launcher import Launcher as _la


class IsoThread:
    threads = []

    @classmethod
    def join(cls):
        """Equal to Thread.join"""
        from timeit import default_timer
        temp = default_timer()
        for i in cls.threads:
            i.wait()
        return temp - default_timer()
    @classmethod
    def clean_all(cls):
        """Clean all the temporary files."""
        for i in cls.threads:
            i.clean()

    def __init__(self, target, name='Thread No.'+str(len(threads)), args: tuple = None,
                 kwargs: dict = None, required_module: list = None):
        """This is true threading. The reason why it is called
        "IsoThread" is that the thread it creates is completely
        separated from the main thread. Therefore it cannot
        print or do modify the variable in the main thread. This
        is used for background process, which needs to be
        separated from the main thread and run quietly."""
        self.name = name
        self.src_name = 'src_' + name + ".func"
        self.run = target
        self.args = args
        self.kwargs = kwargs
        self.mds = required_module
        self.status = 'waiting'
        IsoThread.threads.append(self)

    def _prepare(self):
        _dill.dump(self.run, open(self.src_name, 'wb'))
        self.status = 'starting'
        return _la(self.src_name, name=self.name, args=self.args, kwargs=self.kwargs, required=self.mds)

    def start(self):
        """Start the thread(process)."""
        self._pcs = self._prepare()
        self._pcs.prepare()
        self._pcs.start()
        self.status = 'running'

    def wait(self):
        """Wait and return the exit code."""
        return self._pcs.wait()

    @property
    def finished(self):
        """Finished or not."""
        if self._pcs.finished:
            self.status = 'finished'
            return True
        return False

    @property
    def result(self):
        """Result, will raise ProcessError if not
        finished yet or some exception occurred."""
        import pyil.shared._coll
        try:
            temp = self._pcs.result
            self.status = 'finished'
            del IsoThread.threads[IsoThread.threads.index(self)]
            return temp
        except pyil.shared._coll.ProcessError:
            self.clean()
            self.status = 'paused'
            raise

    def terminate(self) -> 0:
        """terminate the thread."""
        self._pcs.end()
        self.status = 'ended'
        del IsoThread.threads[IsoThread.threads.index(self)]
        return 0

    def clean(self) -> int:
        """Clean up the file being created."""
        self._pcs.clean()
        try:
            _os.unlink(self.src_name)
        except FileNotFoundError:
            self.status = 'cleaned'
            return 1
        self.status = 'cleaned'
        return 0

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.clean()

    def __str__(self):
        return self.name + ": " + self.status



import subprocess as _sp
import sys as _sys
from . import *
__license__ = 'MIT'
__author__ = 'G.M'
__email__ = 'G.Mpydev@gmail.com'
__quote__ = """Loop and recursion work together will produce
better result and better than any of the previous."""
__note__ = """I'm considering to add support for multiprocessing
module for all the search methods, since it is pretty slow if
you have a bunch of files on your computer, though the performance
if o(n)"""

def reinstall():
    """Make sure you have pip added to the environment variable."""
    _sp.Popen(r'resources/reinstall.bat')
    _sys.exit(0)



class open:
    def __new__(cls, file_type: str, *args, **kwargs):
        from .file import TextFile, BinaryFile
        temp = {'text':   TextFile,
                'binary': BinaryFile}
        assert file_type.lower() in temp, "Pyil doesn't currently support this type of file."
        return temp.get(file_type.lower())

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



from pyil.enum.modifiable import divide as _div
from pyil.enum.variable import auto_process


def pmap(func, iterable, process=auto_process) -> iter:
    """Returns a PMap object. The usage is similar to the
    map function and does the same work except that this
    one is multi-threaded."""
    from .thread import IsoThread as itd
    if str(type(process)) == "class <'function'>":
        process = process(len(iterable))
        iterable = _div(iterable, process)
    else:
        iterable = _div(iterable, process)
    temp2 = 0

    def temp(l, fun):
        return list(map(fun, l))

    for i in iterable:
        temp1 = itd(temp, 'mappingProcess ' + str(temp2), (i, func))
        temp1.start()
        temp2 += 1
    itd.join()
    temp = (j for i in itd.threads for j in i.result)
    itd.clean_all()
    return temp

def pfilter(func, iterable:iter, true_only=False):
    """Uses pmap() to filter the result, same as
    the built-in filter function."""
    for i in pmap(func,iterable):
        if true_only:
            if i is True:
                yield i
        else:
            if i is not False:
                yield i





import pickle as _pickle

def write_to_binary(obj, file):
    """using pickle.dump to convert to binary file.
    If you want to change the method, do:
    write_to_binary=your_function
    make sure your the order of the parameter of your function
    is same as this one."""
    _pickle.dump(obj, file)


def read_binary_file(file):
    """Uses pickle.load() to extract the string from
    the binary file, use read_binary=your_function to
    modify it. It will accept 1 parameter which the
    file object it's going to get the information."""
    return _pickle.load(file)

def divide(l:list, interval:int=4):
    """Yield successive n-sized chunks from l. If you
    modify it, please make sure it receives the same
    arguments in the same order and return a list or
    generator."""
    for i in range(0, len(l), interval):
        yield l[i:i + interval]




import os, re


class Path:
    def __init__(self, path):
        """Create a Path object to represent a
        file path. Made for all the file classes
        in order to make them easier to usee."""
        self.__path = os.path.abspath(path)
        self.__name = os.path.basename(path)
        print(self.__name)

    def __getattribute__(self, item):
        if item == 'name' and self.__name == '':
            raise AttributeError("AttributeError: 'Path' object has no attribute 'name'")
        return super().__getattribute__(item)

    def path(self, folder=False):
        return re.sub(self.__name, '', self.__path) if folder else self.__path

    def name(self, no_ext=False):
        return re.sub('\.\w+', '', self.__name) if no_ext else self.__name

    @property
    def exist(self):
        return os.path.exists(self.__path)

    @property
    def size(self):
        return os.path.getsize(self.__path)
    def full(self,no_ext=False):
        print(self.path(folder=True)+self.name(no_ext=True))
        return self.path(folder=True)+self.name(no_ext=True)if no_ext else self.__path

    def __str__(self):
        return self.path()



from getpass import getuser as __gu
import os as __os
import multiprocessing as __mp
true = True
"""Just the value True without caps."""
false = False
"""Just the value False without caps."""
locked=True
"""Values for pyil.shared.LockVar's parameter "state"."""
unlocked=False
"""Values for pyil.shared.LockVar's parameter "state"."""
common_path=[r'C:\Program Files',r'C:\Program Files (x86)', __os.path.join(r'C:\Users', __gu())]
"""The paths where files are likely to be found."""
cpu_count=__mp.cpu_count()
"""The number of cpu(cores) your computer have."""


def auto_process(length, current=cpu_count+1):
    """Pass it to pyil.shared.threading.map
    function to let it decide how many process
    best fit your computer."""
    return current if length%current>current/2or length%\
    current==0else auto_process(length,current=current-1)
pythonPath=r'C:\Users\ryan_\AppData\Local\Programs\Python\Python35\python.exe'
pythonPath2=r'C:\Users\ryan_\AppData\Local\Programs\Python\Python35\pythonw.exe'




import time as _time
class LockVar:
    def __init__(self, obj, default_state=None, block_access: bool = True):
        """Create a variable with a lock, will raise
        assertion error when trying to modify the
        value when locked."""
        self._obj = obj
        self._locked = default_state
        self._blocked = block_access

    @property
    def locked(self):
        """Returns weather the variable is locked."""
        return self._locked

    @property
    def value(self):
        """Returns the value of this obj. Will wait
        until the variable is unlocked if block_access
        if set to True."""
        if self._blocked:
            while True:
                if not self._locked:
                    return self._obj
        return self._obj

    def modify(self, new_value):
        """Modify the value."""
        assert self._locked is False, "Can't modify a variable while it is locked."
        self._obj = new_value
        return self.value

    def lock(self, interval: int = 0):
        """Wait until the object is unlocked, then lock it.
        interval is the time to sleep between each check."""
        while True:
            _time.sleep(interval)
            if not self._locked:
                self._locked = True
                break

    def unlock(self):
        """Unlock the variable."""
        assert self._locked is True, 'Variable already unlocked.'
        self._locked = True



from binaryornot.check import is_binary as _is_binary
from typing import NewType
from pyil.shared._coll import BaseFile as __base
from pyil.shared._coll import FileError
from .__util import Path as __path
__doc__ = '''This module contains all the
functions for text file in pyil.'''

PathObj=NewType('Pathobj',__path)
class TextFile(__base):
    def __init__(self, name: PathObj, res_name: PathObj, **attri):
        """name >- the path of the original file(text file)
        res_name >- the path of the destination file
        (include file name and the extension for the first
        one but no extension for the second one)"""
        if _is_binary(str(name)):  # Check if it is text file.
            raise FileError('Cannot open binary files: %s' % name)
        super().__init__()  # Call super class so I don't have to declare the variable again.
        if len(attri) >= 1:
            for key, item in attri.items():
                if key == 'next_ele_token':
                    self._ne = attri['next_ele_token']
                elif key == 'next_line_token':
                    self._nl = attri['next_line_token']
                elif key == 'delete_original_file':
                    self._dof = attri['delete_original_file']
                else:
                    raise KeyError('No such attributes.')
        self._path = res_name
        self._opath = name
        self._ftext = self._get_ftext()

    def convert_to_binary_file(self):
        """Convert the file into binary file with extension using pickle.dump."""
        from ..enum.modifiable import write_to_binary
        write_to_binary(self.text, open(self._path.full(no_ext=True) + '.dat', 'wb'))


class BinaryFile(__base):
    def __init__(self, name: PathObj, res_name: PathObj, **attri):
        """name >- the path of the original file(binary file, uses read_binary() to read)
        res_name >- the path of the destination file
        (include file name and the extension for the first
        one but no extension for the second one)"""
        if not _is_binary(str(name)):  # Check if it is binary file.
            raise FileError('Cannot open text files: %s' % name)
        super().__init__()  # Call super class.
        if len(attri) >= 1:  # Get attributes.
            for key, item in attri.items():
                if key == 'next_ele_token':
                    self._ne = attri['next_ele_token']
                elif key == 'next_line_token':
                    self._nl = attri['next_line_token']
                elif key == 'delete_original_file':
                    self._dof = attri['delete_original_file']
                else:
                    raise KeyError('No such attributes.')
        self._path = res_name
        self._opath = name
        self._ftext = self._get_ftext()

    @property
    def text(self):
        """The text in the file, change read_binary_file()
         to change the way it reads the text."""
        from ..enum.modifiable import read_binary_file
        return read_binary_file(open(str(self._opath), 'rb'))

    def write(self, obj):
        """Uses write_to_binary function to
        write."""
        from ..enum.modifiable import write_to_binary
        write_to_binary(obj, open(str(self._opath), 'wb'))
        self._ftext = self._get_ftext()
        return len(obj)

    def append(self, obj: str):
        """Uses write_to_binary function to
        write."""
        from ..enum.modifiable import write_to_binary
        write_to_binary(obj, open(str(self._opath), 'ab'))
        self._ftext = self._get_ftext()
        return len(self.text)

    def convert_to_text_file(self):
        """Convert to text file."""
        f = open(self._path.full(no_ext=True)+'.txt', 'w')
        f.write(self.text)
        f.close()


class CsvFile:
    def __init__(self, name:str, res_name:str):
        self.__path = name
        self.__respath = res_name

    @property
    def text(self):
        return open(self.__path).read()

    @property
    def csvformatted_text(self):
        import csv
        return list(csv.reader(open(self.__path)))

    def convert_to_text(self, pure_text=True, next_ele_token=' ', next_line_token='\n'):
        if pure_text:
            with open(self.__respath + '.txt', 'w')as f:
                f.write(self.text)
        else:
            with open(self.__respath + '.txt', 'a')as f:
                for i in self.csvformatted_text:
                    f.write(next_ele_token.join(i))
                    f.write(next_line_token)

    def convert_to_binary(self, pure_text=True, next_ele_token=' ', next_line_token='\n'):
        from ..enum.modifiable import write_to_binary
        if pure_text:
            write_to_binary(self.text, open(self.__respath + '.dat', 'wb'))
        else:
            for i in self.csvformatted_text:
                write_to_binary(i, open(self.__respath + '.dat', 'a'))
                write_to_binary(next_line_token, open(self.__respath + '.dat', 'a'))

    def convert_to_xlsx(self):
        """Convert the file into Microsoft spread sheet."""
        import openpyxl
        self.__wb = openpyxl.Workbook()
        self.__sheet = self.__wb.get_sheet_by_name("Sheet")
        temp = 1
        for i in self.csvformatted_text:
            self.__write_row(i, temp)
            temp += 1
        self.__wb.save(self.__path + '.xlsx')
        del self.__wb
        del self.__sheet

    def __write_row(self, list_to_write, start_row, start_column=1):
        try:
            self.__sheet.cell(row=start_row, column=start_column).value = list_to_write[start_column - 1]
            return self.__write_row(list_to_write, start_row, start_column=start_column + 1)
        except IndexError:
            return

    def convert_to_json(self, pure_text=True):
        import json
        if pure_text:
            json.dump(self.text, open(self.__respath + '.json', 'w'))
        else:
            json.dump(self.csvformatted_text, open(self.__respath + '.json', 'w'))

    def write(self, iterable: iter):
        import csv
        return csv.writer(open(self.__respath + '.csv', 'w')).writerow(iterable)

    def append(self, iterable: iter):
        import csv
        return csv.writer(open(self.__respath + '.csv', 'a')).writerow(iterable)



import re,random
from .error import FileError
def cli(ext: str):
    raise FileError('cannot open file with this extension: %s.' % ext)


def switch_bool(v: bool):
    return False if v else True


def compare(n1: str, n2: str, i=False):
    return re.match(n2.lower(), n1.lower()) is not None if i else re.match(n2, n1) is not None

def random_word(length:int, word_only:bool=False)->str:
    """Generates random letter combinations."""
    if not word_only:
        temp = ('a', 'b', 'c', 'd', 'e', 'f', '1', '2',
                '3', '4','5', '0', 'A', 'h', 'H',
                'B', 'C', 'D', 'E', 'F', '.', ',',
                '?', '<', '>', '!', 'x', 'X', 'G.M')
        temp2 = ''
        for i in range(length):
            temp2 += (random.choice(temp))
        return temp2
    temp = ('a', 'b', 'c', 'd', 'e', 'f', '1', '2', '3', '4', '5', '0', 'A', 'h', 'H',
            'B', 'C', 'D', 'E', 'F', 'x', 'X')
    temp2 = ''
    for i in range(length):
        temp2 += (random.choice(temp))
    return temp2



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import re, sys, pip, platform, time, shutil, os

v = '0.6.6.7'
avi = {'x32_34':   r'resources\lxml-3.6.4-cp34-cp34m-win32.whl',
       'AMD64_34': r'resources\lxml-3.6.4-cp34-cp34m-win_amd64.whl',
       'x32_35':   r'resources\lxml-3.6.4-cp35-cp35m-win32.whl',
       'AMD64_35': r'resources\lxml-3.6.4-cp35-cp35m-win_amd64.whl',
       'none':     """I can't find the version of lxml wheel that fit this platform
        or this version of python, that means you may not be able to use the
        convert_to_docx() function. However, if you figure out a way to install
        lxml and did it, reinstall pyil-converter or call reinstall function."""}
ire = ['Python-Docx', 'binaryornot', 'openpyxl', 'dill']


def install(file):
    if file.endswith('.whl'):
        try:
            temp5 = pip.main(['install', file])
            if temp5 != 0:
                raise Exception()
            print('Successfully installed lxml!')
            print('Will now continued the installation of pyil-converter.')
            return
        except Exception:
            pass
    print(file)
    del ire[0]
    time.sleep(3)


def what():
    with open('CHANGELOG')as file:
        temp6 = file.read()
        tempre = re.compile('v' + v + '.+', flags=re.DOTALL)
        tempm = tempre.search(temp6)
        print('\n\n', tempm.group(0), '\n\n')


def read(name):
    with open(name) as file:
        return file.read()


what()

final = ''
temp1 = platform.machine()
temp2 = re.compile('x86')
temp2.sub('AMD64', temp1)
final += temp1
temp1 = sys.version
temp2 = re.compile(r'\d+(\.\d\.)\d+')
temp3 = temp2.search(temp1)
temp4 = temp3.group()
temp1 = temp4[0] + temp4[2]
final += '_' + temp1
install(avi.get(final, avi['none']))

try:
    setup(
        name='pyil-converter',
        version=v,
        packages=['pyil'],
        license='MIT',
        author='G.M',
        author_email='G.Mpydev@gmail.com',
        description='Convert files to the format you want!',
        long_description=read('README.rst'),
        install_requires=ire.copy(),
        include_package_data=True,
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python :: 3 :: Only',
            'Operating System :: Microsoft :: Windows',
            'Intended Audience :: Developers',
            'Intended Audience :: Information Technology',
            'Natural Language :: English',
            'Topic :: System :: Filesystems',
            'Topic :: System :: Archiving :: Packaging',
            'Topic :: Utilities',
            'License :: OSI Approved :: MIT License'
        ],
        platforms='win32',
        keywords=['file', 'format', 'extension', 'reformatting',
                  'converting', 'data'],
        url=r'https://wasted123.github.io/pyil/'
    )
    print('Installation succeeded.')

except Exception as msg:
    print('\n')
    print('An unexpected exception happened.')
    print()
    print(str(msg))
    print('\n')
    print('Please read the exception and google it.')
finally:
    if not os.path.exists(r'C:\ind'):
        print('Finding python path.')
        from pyil.util import search_file

        result = [search_file('python.exe', caseI=True), search_file('pythonw.exe', caseI=True)]
        content = open(r'.\pyil\enum\variable.py').readlines()
        content = content[:-3]
        print('Done!\nModifying source code.')
        with open(r'.\pyil\enum\variable.py', 'w')as f:
            for i in content:
                f.write(i)
            print("pythonPath=r'" + result[0] + "'", file=f)
            print("pythonPath2=r'" + result[1] + "'", file=f)
            print(file=f)

    print('Removing source package of lxml.')
    if not os.path.exists(r'C:\ind'):
        shutil.rmtree(r'.\resources', ignore_errors=True)
    print('Process completed, you can now close this window.')



import os as __os

from .shared._coll import simplify_str as convert_string
from .enum.variable import false, true


def search_file(name, *path, **option):
    """Search the given file name(re pattern) in the given path, if path
    is none, will search in C:\\(It will search in the order
    you let it to). If enable list_all, it will return a
    tuple containing all result."""
    from .shared._coll import compare
    if len(path) == 0:
        path = 'C:\\'
    ci, topdown, list_all = false, true, false
    size = float('inf')
    for key, value in option.items():  # Get options.
        if key == 'caseI':
            ci = option['caseI']
        elif key == 'topdown':
            topdown = option['topdown']
        elif key == 'list_all':
            list_all = option['list_all']
        elif key == 'size':
            size = option['size']
        else:
            raise KeyError('No such option.')
    temp = []
    for ci in path:
        for current_folder, subfolder, subfile in __os.walk(ci, topdown=topdown):
            for fname in subfile:
                if compare(fname, name, i=ci):
                    if list_all and __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                        temp.append(__os.path.abspath(__os.path.join(current_folder, fname)))
                    elif __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                        return __os.path.abspath(__os.path.join(current_folder, fname))
    if len(temp) > 0:
        return tuple(temp)
    return None


def search_folder(name, *path, **option):
    """Search the given folder name(re pattern) in the given path, if path
    is none, will search in C:\\(It will search in the order
    you let it to). If enable list_all, it will return a
    tuple containing all result."""
    from .shared._coll import compare
    if len(path) == 0:
        path = 'C:\\'
    ci, topdown, list_all, empty = false, true, false, false
    size = float('inf')
    for key, value in option.items():
        if key == 'caseI':
            ci = option['caseI']
        elif key == 'topdown':
            topdown = option['topdown']
        elif key == 'list_all':
            list_all = option['list_all']
        elif key == 'size':
            size = option['size']
        elif key == 'ignore_empty_folder':
            empty = option['ignore_empty_folder']
        else:
            raise KeyError('No such option.')
    temp = []
    for ci in path:
        for current_folder, subfolder, subfile in __os.walk(ci, topdown=topdown):
            for fname in subfolder:
                if compare(fname, name, i=ci):
                    if empty == false:
                        if list_all and __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                            temp.append(__os.path.abspath(__os.path.join(current_folder, fname)))
                        elif __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                            return __os.path.abspath(__os.path.join(current_folder, fname))
                    else:
                        if __os.listdir(__os.path.join(current_folder, fname)) == '':
                            continue
                        if list_all and __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                            temp.append(__os.path.abspath(__os.path.join(current_folder, fname)))
                        elif __os.path.getsize(__os.path.join(current_folder, fname)) / 1024 <= size:
                            return __os.path.abspath(__os.path.join(current_folder, fname))
    if len(temp) > 0:
        return tuple(temp)
    return None


def parselist(string: str, caseI=False, file=False):
    """Parse a string into list: '[1,2,3]'>-[1,2,3].
    If their is some problem with the string containing
    the list, it would probably give an incorrect result."""
    if file:
        from .shared._coll import random_word
        import importlib
        temp = "tempList" + random_word(4, True)
        with open(temp + ".py", 'w')as f:
            print("templ=", string, file=f)
        try:
            mod = importlib.import_module(temp)
        except SyntaxError:
            raise SyntaxError("Something is wrong with the list you are using.")
        else:
            temp1 = mod.templ
            __os.unlink(temp + '.py')
            return temp1

    from .shared._coll import switch_bool

    string = string[1:-1] + ','
    in_quote, reading = false, false
    part = ''
    final = []

    for i in string:  # Loop through the string to extract information.
        if in_quote == false and i == ']':
            part += i
            part = (parselist(part))  # Call itself again to handle that list
            reading = false

        elif (in_quote == false and i == '[') or reading == true:
            reading = true
            part += i
            continue
        elif i == "'" or i == '"':
            try:
                if i[string.index(i) + 1] == ',' or i[string.index(i) - 1] == ',':
                    in_quote = switch_bool(in_quote)
            except IndexError:
                pass

        elif i == ',' and in_quote == false:
            final.append(part)
            part = ''

        elif in_quote:
            part += i
        else:
            part = convert_string(i, casei=caseI).simplify()

    return final
